@@ -36,11 +36,11 @@ from robot.api.parsing import get_model
 from robot.errors import VariableError
 from robot.output import LOGGER
 from robot.running import EXECUTION_CONTEXTS, Keyword, TestCase, TestSuite
-from robot.running.userkeyword import UserKeywordHandler
 from robot.utils import NormalizedDict
 from robot.variables import evaluate_expression
+
 from robotcode.core.event import event
-from robotcode.core.logging import LoggingDescriptor
+from robotcode.core.utils.logging import LoggingDescriptor
 from robotcode.robot.utils import get_robot_version
 
 from .dap_types import (
@@ -72,6 +72,13 @@ from .dap_types import (
     VariablePresentationHint,
 )
 
+if get_robot_version() >= (7, 0):
+    from robot.running.invalidkeyword import (
+        InvalidKeywordRunner as UserKeywordHandler,
+    )
+else:
+    from robot.running.userkeyword import UserKeywordHandler
+
 if get_robot_version() >= (6, 1):
 
     def internal_evaluate_expression(expression: str, variable_store: Any) -> Any:
@@ -241,10 +248,10 @@ else:
         def __init__(self) -> None:
             self.steps: List[Any] = []
 
-        def start_keyword(self, data: "running.Keyword", result: "result.Keyword") -> None:
+        def start_keyword(self, data: running.Keyword, result: result.Keyword) -> None:
             self.steps.append(data)
 
-        def end_keyword(self, data: "running.Keyword", result: "result.Keyword") -> None:
+        def end_keyword(self, data: running.Keyword, result: result.Keyword) -> None:
             self.steps.pop()
 
 
@@ -305,7 +312,6 @@ class Debugger:
         self.stop_on_entry = False
         self._debug = True
         self.terminated = False
-        self.terminated_requested = False
         self.attached = False
         self.path_mappings: List[PathMapping] = []
         self.server_loop: Optional[asyncio.AbstractEventLoop] = None
@@ -332,7 +338,10 @@ class Debugger:
     def state(self, value: State) -> None:
         # if state is changed, do nothing and wait a little bit to avoid busy loop
 
-        if self._state == State.Paused and value not in [State.Paused, State.CallKeyword]:
+        if self._state == State.Paused and value not in [
+            State.Paused,
+            State.CallKeyword,
+        ]:
             self._variables_cache.clear()
             self._variables_object_cache.clear()
             self._evaluate_cache.clear()
@@ -373,9 +382,6 @@ class Debugger:
     def robot_output_file(self, value: Optional[str]) -> None:
         self._robot_output_file = value
 
-    def terminate_requested(self) -> None:
-        self.terminated_requested = True
-
     def terminate(self) -> None:
         self.terminated = True
 
@@ -392,7 +398,10 @@ class Debugger:
                 self.send_event(
                     self,
                     ContinuedEvent(
-                        body=ContinuedEventBody(thread_id=self.main_thread.ident, all_threads_continued=True)
+                        body=ContinuedEventBody(
+                            thread_id=self.main_thread.ident,
+                            all_threads_continued=True,
+                        )
                     ),
                 )
 
@@ -424,7 +433,10 @@ class Debugger:
             raise InvalidThreadIdError(thread_id)
 
         with self.condition:
-            if self.full_stack_frames and self.full_stack_frames[0].type in ["TEST", "SUITE"]:
+            if self.full_stack_frames and self.full_stack_frames[0].type in [
+                "TEST",
+                "SUITE",
+            ]:
                 self.requested_state = RequestedState.StepIn
             else:
                 self.requested_state = RequestedState.Next
@@ -447,7 +459,10 @@ class Debugger:
             self.condition.notify_all()
 
     def step_in(
-        self, thread_id: int, target_id: Optional[int] = None, granularity: Optional[SteppingGranularity] = None
+        self,
+        thread_id: int,
+        target_id: Optional[int] = None,
+        granularity: Optional[SteppingGranularity] = None,
     ) -> None:
         if self.main_thread is None or thread_id != self.main_thread.ident:
             raise InvalidThreadIdError(thread_id)
@@ -485,7 +500,7 @@ class Debugger:
             self.condition.notify_all()
 
     @event
-    def send_event(sender: Any, event: Event) -> None:  # NOSONAR
+    def send_event(sender: Any, event: Event) -> None:
         ...
 
     def set_breakpoints(
@@ -504,10 +519,16 @@ class Debugger:
             self.breakpoints.pop(path)
         elif path:
             self.breakpoints[path] = result = BreakpointsEntry(
-                tuple(breakpoints) if breakpoints else (), tuple(lines) if lines else ()
+                tuple(breakpoints) if breakpoints else (),
+                tuple(lines) if lines else (),
             )
             return [
-                Breakpoint(id=id(v), source=Source(path=str(path)), verified=True, line=v.line)
+                Breakpoint(
+                    id=id(v),
+                    source=Source(path=str(path)),
+                    verified=True,
+                    line=v.line,
+                )
                 for v in result.breakpoints
             ]
 
@@ -586,7 +607,12 @@ class Debugger:
                             hit = False
                             try:
                                 vars = EXECUTION_CONTEXTS.current.variables.current
-                                hit = bool(internal_evaluate_expression(vars.replace_string(point.condition), vars))
+                                hit = bool(
+                                    internal_evaluate_expression(
+                                        vars.replace_string(point.condition),
+                                        vars,
+                                    )
+                                )
                             except (SystemExit, KeyboardInterrupt):
                                 raise
                             except BaseException:
@@ -643,7 +669,13 @@ class Debugger:
                             ),
                         )
 
-    def process_end_state(self, status: str, filter_id: Set[str], description: str, text: Optional[str]) -> None:
+    def process_end_state(
+        self,
+        status: str,
+        filter_id: Set[str],
+        description: str,
+        text: Optional[str],
+    ) -> None:
         if self.state == State.Stopped:
             return
 
@@ -705,7 +737,10 @@ class Debugger:
                         self.send_event(
                             self,
                             ContinuedEvent(
-                                body=ContinuedEventBody(thread_id=self.main_thread.ident, all_threads_continued=True)
+                                body=ContinuedEventBody(
+                                    thread_id=self.main_thread.ident,
+                                    all_threads_continued=True,
+                                )
                             ),
                         )
                     continue
@@ -714,14 +749,14 @@ class Debugger:
 
     def start_output_group(self, name: str, attributes: Dict[str, Any], type: Optional[str] = None) -> None:
         if self.group_output:
-            source = attributes.get("source", None)
-            line_no = attributes.get("lineno", None)
+            source = attributes.get("source")
+            line_no = attributes.get("lineno")
 
             self.send_event(
                 self,
                 OutputEvent(
                     body=OutputEventBody(
-                        output=f"\u001b[38;5;14m{(type +' ') if type else ''}\u001b[0m{name}\n",
+                        output=f"\u001b[38;5;14m{(type + ' ') if type else ''}\u001b[0m{name}\n",
                         category=OutputCategory.CONSOLE,
                         group=OutputGroup.START,
                         source=Source(path=str(self.map_path_to_client(source))) if source else None,
@@ -733,8 +768,8 @@ class Debugger:
 
     def end_output_group(self, name: str, attributes: Dict[str, Any], type: Optional[str] = None) -> None:
         if self.group_output:
-            source = attributes.get("source", None)
-            line_no = attributes.get("lineno", None)
+            source = attributes.get("source")
+            line_no = attributes.get("lineno")
 
             self.send_event(
                 self,
@@ -787,7 +822,7 @@ class Debugger:
 
         self.full_stack_frames.appendleft(result)
 
-        if type in ["KEYWORD"] and source is None and line is None and column is None:
+        if type == "KEYWORD" and source is None and line is None and column is None:
             return result
 
         if type in ["SUITE", "TEST"]:
@@ -815,7 +850,7 @@ class Debugger:
     ) -> None:
         self.full_stack_frames.popleft()
 
-        if type in ["KEYWORD"] and source is None and line is None and column is None:
+        if type == "KEYWORD" and source is None and line is None and column is None:
             return
 
         if type in ["SUITE", "TEST"]:
@@ -835,7 +870,7 @@ class Debugger:
             self.debug_logger = DebugLogger()
             LOGGER.register_logger(self.debug_logger)
 
-        source = attributes.get("source", None)
+        source = attributes.get("source")
         line_no = attributes.get("lineno", 1)
         longname = attributes.get("longname", "")
         status = attributes.get("status", "")
@@ -861,7 +896,12 @@ class Debugger:
 
                 self.wait_for_running()
             elif entry.source:
-                self.process_start_state(entry.source, entry.line if entry.line is not None else 0, entry.type, status)
+                self.process_start_state(
+                    entry.source,
+                    entry.line if entry.line is not None else 0,
+                    entry.type,
+                    status,
+                )
 
                 self.wait_for_running()
 
@@ -875,19 +915,19 @@ class Debugger:
                         status,
                         {"failed_suite"},
                         "Suite failed.",
-                        f"Suite failed{f': {v}' if (v:=attributes.get('message', None)) else ''}",
+                        f"Suite failed{f': {v}' if (v := attributes.get('message')) else ''}",
                     )
 
                 self.wait_for_running()
 
-        source = attributes.get("source", None)
+        source = attributes.get("source")
         line_no = attributes.get("lineno", 1)
         type = "SUITE"
 
         self.remove_stackframe_entry(name, type, source, line_no)
 
     def start_test(self, name: str, attributes: Dict[str, Any]) -> None:
-        source = attributes.get("source", None)
+        source = attributes.get("source")
         line_no = attributes.get("lineno", 1)
         longname = attributes.get("longname", "")
         status = attributes.get("status", "")
@@ -897,7 +937,12 @@ class Debugger:
         entry = self.add_stackframe_entry(name, type, source, line_no, longname=longname)
 
         if self.debug and entry.source:
-            self.process_start_state(entry.source, entry.line if entry.line is not None else 0, entry.type, status)
+            self.process_start_state(
+                entry.source,
+                entry.line if entry.line is not None else 0,
+                entry.type,
+                status,
+            )
 
             self.wait_for_running()
 
@@ -910,12 +955,12 @@ class Debugger:
                     status,
                     {"failed_test"},
                     "Test failed.",
-                    f"Test failed{f': {v}' if (v:=attributes.get('message', None)) else ''}",
+                    f"Test failed{f': {v}' if (v := attributes.get('message')) else ''}",
                 )
 
                 self.wait_for_running()
 
-        source = attributes.get("source", None)
+        source = attributes.get("source")
         line_no = attributes.get("lineno", 1)
         longname = attributes.get("longname", "")
         type = "TEST"
@@ -924,11 +969,11 @@ class Debugger:
 
     def start_keyword(self, name: str, attributes: Dict[str, Any]) -> None:
         status = attributes.get("status", "")
-        source = attributes.get("source", None)
-        line_no = attributes.get("lineno", None)
+        source = attributes.get("source")
+        line_no = attributes.get("lineno")
         type = attributes.get("type", "KEYWORD")
-        libname = attributes.get("libname", None)
-        kwname = attributes.get("kwname", None)
+        libname = attributes.get("libname")
+        kwname = attributes.get("kwname")
 
         handler: Any = None
         if type in ["KEYWORD", "SETUP", "TEARDOWN"]:
@@ -940,10 +985,17 @@ class Debugger:
                 pass
 
         entry = self.add_stackframe_entry(
-            kwname, type, source, line_no, handler=handler, libname=libname, kwname=kwname, longname=name
+            str(kwname),
+            type,
+            source,
+            line_no,
+            handler=handler,
+            libname=libname,
+            kwname=kwname,
+            longname=name,
         )
 
-        if status == "NOT RUN" and type not in ["IF"]:
+        if status == "NOT RUN" and type != "IF":
             return
 
         if self.debug and entry.source and entry.line is not None:
@@ -1000,7 +1052,10 @@ class Debugger:
             return False
 
         for pattern in branch.patterns:
-            if matcher(error, EXECUTION_CONTEXTS.current.variables.replace_string(pattern)):
+            if matcher(
+                error,
+                EXECUTION_CONTEXTS.current.variables.replace_string(pattern),
+            ):
                 return True
 
         return False
@@ -1020,7 +1075,7 @@ class Debugger:
         return True
 
     def end_keyword(self, name: str, attributes: Dict[str, Any]) -> None:
-        type = attributes.get("type", None)
+        type = attributes.get("type")
         if self.debug:
             status = attributes.get("status", "")
 
@@ -1042,10 +1097,10 @@ class Debugger:
 
                 self.wait_for_running()
 
-        source = attributes.get("source", None)
-        line_no = attributes.get("lineno", None)
+        source = attributes.get("source")
+        line_no = attributes.get("lineno")
         type = attributes.get("type", "KEYWORD")
-        kwname = attributes.get("kwname", None)
+        kwname = attributes.get("kwname")
 
         handler: Any = None
         if type in ["KEYWORD", "SETUP", "TEARDOWN"]:
@@ -1056,7 +1111,7 @@ class Debugger:
             except BaseException:
                 pass
 
-        self.remove_stackframe_entry(kwname, type, source, line_no, handler=handler)
+        self.remove_stackframe_entry(str(kwname), type, source, line_no, handler=handler)
 
     def set_main_thread(self, thread: threading.Thread) -> None:
         self.main_thread = thread
@@ -1064,7 +1119,12 @@ class Debugger:
     def get_threads(self) -> List[Thread]:
         main_thread = self.main_thread or threading.main_thread()
 
-        return [Thread(id=main_thread.ident if main_thread.ident else 0, name=main_thread.name or "")]
+        return [
+            Thread(
+                id=main_thread.ident if main_thread.ident else 0,
+                name=main_thread.name or "",
+            )
+        ]
 
     WINDOW_PATH_REGEX: ClassVar = re.compile(r"^(([a-z]:[\\/])|(\\\\)).*$", re.RegexFlag.IGNORECASE)
 
@@ -1102,7 +1162,10 @@ class Debugger:
 
     def source_from_entry(self, entry: StackFrameEntry) -> Optional[Source]:
         if entry.source is not None and entry.is_file:
-            return Source(path=str(self.map_path_to_client(entry.source)), presentation_hint="normal")
+            return Source(
+                path=str(self.map_path_to_client(entry.source)),
+                presentation_hint="normal",
+            )
 
         return None
 
@@ -1167,7 +1230,13 @@ class Debugger:
 
     RE_FILE_LINE_MATCHER = re.compile(r".+\sin\sfile\s'(?P<file>.*)'\son\sline\s(?P<line>\d+):.*")
 
-    def _send_log_event(self, timestamp: str, level: str, msg: str, category: Union[OutputCategory, str]) -> None:
+    def _send_log_event(
+        self,
+        timestamp: str,
+        level: str,
+        msg: str,
+        category: Union[OutputCategory, str],
+    ) -> None:
         current_frame = self.full_stack_frames[0] if self.full_stack_frames else None
         source = (
             Source(path=str(self.map_path_to_client(current_frame.source)))
@@ -1235,7 +1304,7 @@ class Debugger:
                         variables_reference=entry.local_id(),
                     )
                 )
-                if context.variables._test is not None and entry.type in ["KEYWORD"]:
+                if context.variables._test is not None and entry.type == "KEYWORD":
                     result.append(
                         Scope(
                             name="Test",
@@ -1244,7 +1313,10 @@ class Debugger:
                             variables_reference=entry.test_id(),
                         )
                     )
-                if context.variables._suite is not None and entry.type in ["TEST", "KEYWORD"]:
+                if context.variables._suite is not None and entry.type in [
+                    "TEST",
+                    "KEYWORD",
+                ]:
                     result.append(
                         Scope(
                             name="Suite",
@@ -1385,7 +1457,8 @@ class Debugger:
                         result[repr(i)] = self._create_variable(repr(k), v)
                         if i >= 500:
                             result["Unable to handle"] = self._create_variable(
-                                "Unable to handle", "Maximum number of items (500) reached."
+                                "Unable to handle",
+                                "Maximum number of items (500) reached.",
                             )
                             break
 
@@ -1479,7 +1552,11 @@ class Debugger:
                         if splitted:
 
                             def run_kw() -> Any:
-                                kw = Keyword(name=splitted[0], args=tuple(splitted[1:]), assign=tuple(variables))
+                                kw = Keyword(
+                                    name=splitted[0],
+                                    args=tuple(splitted[1:]),
+                                    assign=tuple(variables),
+                                )
                                 return kw.run(evaluate_context)
 
                             result = self.run_in_robot_thread(run_kw)
@@ -1655,7 +1732,11 @@ class Debugger:
         return SetVariableResult(value=repr(value), type=repr(type(value)))
 
     def set_variable(
-        self, variables_reference: int, name: str, value: str, format: Optional[ValueFormat] = None
+        self,
+        variables_reference: int,
+        name: str,
+        value: str,
+        format: Optional[ValueFormat] = None,
     ) -> SetVariableResult:
         entry = next(
             (
@@ -1693,7 +1774,12 @@ class Debugger:
 
         if filter_options is not None:
             for option in filter_options:
-                if option.filter_id in ["failed_keyword", "uncaught_failed_keyword", "failed_test", "failed_suite"]:
+                if option.filter_id in [
+                    "failed_keyword",
+                    "uncaught_failed_keyword",
+                    "failed_test",
+                    "failed_suite",
+                ]:
                     entry = ExceptionBreakpointsEntry(
                         tuple(filters),
                         tuple(filter_options) if filter_options is not None else None,
@@ -1708,7 +1794,11 @@ class Debugger:
         return result or None
 
     def completions(
-        self, text: str, column: int, line: Optional[int] = None, frame_id: Optional[int] = None
+        self,
+        text: str,
+        column: int,
+        line: Optional[int] = None,
+        frame_id: Optional[int] = None,
     ) -> List[CompletionItem]:
         if self.expression_mode:
             return []
